<?php

/*
Keep the original copyright noticei here, because this script is heavyly 
based on work made by the original authors.
*/

/*
Copyright (c) 2007, Till Brehm, projektfarm Gmbh
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of ISPConfig nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


/*
Modifications by Matthias Wamser, ILK Internet GmbH, mw (at) ilk.net :

$Rev:: 00            $:  Revision of last commit
$Author:: mw         $:  Author of last commit
$Date:: 2013-04-18 1#$:  Date of last commit

Source: /usr/local/ispconfig/server/cron_daily.php

Major rewrite for processing remote logs. Don't touch ISPConfig's own
scripts or processes. This one runs completely standalone.
It has to be run on one the of the ISPConfig hosts, who receives the remote
log from the other hosts.
The ISPConfig stats creation is independent from this. Even the type of
stats (webalizer or awstats) doesn't matter. We independently try to 
generate awstats statistics on all websites known to the master db.

Features:
 * Create awstats on remote logged apache logs
 * Update awstats statistics at any time (looking for yesterdays and todays logfile).
 * Generate stats for every alias- and subdomain in separate dirs.
 * Compress yesterdays logfile after processing
*/


// ######################################################################
// Set some vars
// ######################################################################


//////////////////////////////
//  custom vars
$custom_conf_dir = '/etc/custom-ispconfig';
$custom_awstats_conf_dir = $custom_conf_dir.'/awstats';
$custom_log_dir = '/var/log/custom-ispconfig/httpd';
$custom_stats_site = 'stats.foo.bar';

define('SCRIPT_PATH', '/usr/local/ispconfig/server');
require(SCRIPT_PATH."/lib/config.inc.php");
require(SCRIPT_PATH."/lib/app.inc.php");

set_time_limit(0);

//////////////////////////////
// This is the local server id
// make sure server_id is always an int
$conf['server_id'] = intval($conf['server_id']);


// Load required base-classes
$app->uses('ini_parser,file,services,getconf');


$web_config = $app->getconf->get_server_config($conf['server_id'], 'web');

//////////////////////////////
$today = date('Ymd');
$yesterday = date('Ymd',strtotime("-1 day", time()));

// Determine correct month and year for yesterdays and todays logfile because it differs on first of month
// Used for awstats creation of static html pages and creation of awstatsdir
//
// used for todays logfile handling
$awmonth_today = date("m");
$awyear_today = date("Y");
// used for yesterdays logfile handling
$awmonth = date("m");
$awyear = date("Y");
 // on first of month yesterdays logfile is equal to last months logfile
if (date("d") == 1) {
        $lastmonth = date("m")-1;
        $awmonth = sprintf("%02s", $lastmonth);
        // and on first of month of year yesterdays logfile is equal to last years logfile
        if (date("m") == 1) {
                if ($debug) print "[debug] month = ".date("m")."\n";
                $awyear = date("Y")-1;
                $awmonth = "12";
        }
}

$awstats_pl = $web_config['awstats_pl'];
$awstats_buildstaticpages_pl = $web_config['awstats_buildstaticpages_pl'];
$awstats_conf_dir = $web_config['awstats_conf_dir'];

function create_awstats_conf($given_logfile) {
global $debug, $awstats_website_conf_file, $awstats_conf_dir, $custom_awstats_conf_dir, $statsdomain, $awstats_website_conf_file;
// Delete if config exists
if(is_file($awstats_website_conf_file)) {
        unlink($awstats_website_conf_file);
}
// Create awstats.$statsdomain.conf for $given_logfile
if(!is_file($awstats_website_conf_file)) {
	$awstats_conf_file_content = '# Generated by awstats processor script
Include "'.$awstats_conf_dir.'/awstats.conf"
Include "'.$custom_awstats_conf_dir.'/awstats.conf.override"
LogFile="'.$given_logfile.'"
SiteDomain="'.$statsdomain.'"
HostAliases="www.'.$statsdomain.' localhost 127.0.0.1'.'"';
	file_put_contents($awstats_website_conf_file,$awstats_conf_file_content);
	if ($debug) print "[debug] File ".$awstats_website_conf_file." (re)created.\n";
	}
}

	// ######################################################################
	// Get some DB records from local db
	// ######################################################################
	$sql = "SELECT domain, document_root, system_user, system_group FROM web_domain WHERE domain = '$custom_stats_site' AND server_id = ".$conf['server_id'];
	$records = $app->db->queryAllRecords($sql);
	foreach($records as $rec) {
	$stats_site = escapeshellcmd($rec['domain']);
	$stats_site_document_root = escapeshellcmd($rec['document_root']);
	$stats_site_system_user = escapeshellcmd($rec['system_user']);
	$stats_site_system_group = escapeshellcmd($rec['system_group']);

	}	

$custom_stats_web_dir = $stats_site_document_root.'/web/sites';
$custom_stats_htaccess = $stats_site_document_root.'/web/sites/.htaccess';
$stats_htaccess = $stats_site_document_root.'/web/stats/.htaccess';

	//////////////////////////////
	// custom_stats_web_dir creation
	if(!is_dir($custom_stats_web_dir)) {
   		mkdir($custom_stats_web_dir);
	}

        if(!@is_file($custom_stats_htaccess)) {
		if(@is_file($stats_htaccess)) {
	        copy($stats_htaccess,$custom_stats_htaccess);
		}
        }



	// ######################################################################
	// Get DB records from master db
	// ######################################################################
	$sql = "SELECT domain_id, domain, document_root, system_user, system_group, ip_address FROM web_domain WHERE stats_type = 'awstats' OR stats_type= 'webalizer'";
	$records = $app->dbmaster->queryAllRecords($sql);

	// ######################################################################
	// Create awstats statistics
	// ######################################################################

	//////////////////////////////
	// get domain as $rec
	foreach($records as $rec) {
	$domain = escapeshellcmd($rec['domain']);
	$logfile_yesterday = escapeshellcmd($custom_log_dir.'/'.$rec['domain'].'/'.$yesterday.'-access.log');
	$logfile_today = escapeshellcmd($custom_log_dir.'/'.$rec['domain'].'/'.$today.'-access.log');

	///////////////////////
	// Look if yesterdays or todays logfile exist, if none exists continue
        if(!@is_file($logfile_yesterday)) {
        }
        if(!@is_file($logfile_today)) {
        }
        if(!@is_file($logfile_yesterday) && !@is_file($logfile_today)) {
                continue;
        }


	// Don't use IP, if ISPConfig doesn't know about (if there is a loadbalancer in front)
	//$domain_ip = escapeshellcmd($rec['ip_address']);
	//if ($debug) print "[debug] Primary domain = ".$domain." (".$domain_ip.") \n";
	
	
	///////////////////////
	// get all Alias- and Subdomain for Domain as $statsdomain
	$sql = "SELECT domain FROM web_domain WHERE (type = 'alias' OR type = 'subdomain') AND parent_domain_id = ".$rec['domain_id'];
	$aliases = $app->dbmaster->queryAllRecords($sql);
	$statsdomain = '';
	// add parent domain (type vhost) and not domain_ip to array
	$aliases[] = array( "domain" => $domain);
	// Don't add IP, if ISPConfig doesn't know about (if there is a loadbalancer in front)
	//$aliases[] = array( "domain" => $domain_ip);

	///////////////////////
	// BEG: foreach aliases
	foreach ($aliases as $alias) {
		$statsdomain = $alias['domain'];
		$statsdomaindir = $custom_stats_web_dir.'/'.$domain.'/'.$statsdomain;
		$statsdir = $statsdomaindir.'/'.$awyear.'-'.$awmonth;
		$awstats_website_conf_file = $custom_awstats_conf_dir.'/awstats.'.$statsdomain.'.conf';

		// if upper domain dir does not exist, create it
		if(!is_dir($custom_stats_web_dir.'/'.$domain)) {
			mkdir($custom_stats_web_dir.'/'.$domain);
			}

		// if main domain dir does not exist, create it
		if(!is_dir($statsdomaindir)) {
			mkdir($statsdomaindir);
			}


		///////////////////////
		// statsdir creation per $statsdomain
		if(!is_dir($statsdir)) {
			mkdir($statsdir);
			}

		///////////////////////////////////////////////////////////////////////////////
		// BEG: awstats processing for $statsdomain

		$command_yesterday = "$awstats_buildstaticpages_pl -month='$awmonth' -year='$awyear' -update -configdir='$custom_awstats_conf_dir' -config='$statsdomain' -lang=".$conf['language']." -dir='$statsdir' -awstatsprog='$awstats_pl'";
		$command_today = "$awstats_buildstaticpages_pl -month='$awmonth_today' -year='$awyear_today' -update -configdir='$custom_awstats_conf_dir' -config='$statsdomain' -lang=".$conf['language']." -dir='$statsdir' -awstatsprog='$awstats_pl'";

		/////////////////////////////////////////////////////////////////
		// Create statistics if awstats scripts exist and owner matches	
		if($awstats_pl != '' && $awstats_buildstaticpages_pl != '' && fileowner($awstats_pl) == 0 && fileowner($awstats_buildstaticpages_pl) == 0) {
			if(@is_file($logfile_yesterday)) {
				if ($debug) print "[debug] Processing logfile_yesterday\n";
				create_awstats_conf("$logfile_yesterday");
				exec($command_yesterday);
				$app->log('command_yesterday: '.$command_yesterday,LOGLEVEL_DEBUG);
			}
                	if(@is_file($logfile_today)) {
				if ($debug) print "[debug] Processing logfile_today\n";
				create_awstats_conf("$logfile_today");
				exec($command_today);
				$app->log('command_today: '.$command_today,LOGLEVEL_DEBUG);
                	}

			if(is_file($statsdir.'/index.html')) unlink($statsdir.'/index.html');
			rename($statsdir.'/awstats.'.$statsdomain.'.html',$statsdir.'/awsindex.html');
			// we use indexing in the first directory - means easier navigation
			//copy($custom_conf_dir."/templates/awstats_index.php.master",$custom_stats_web_dir."/".$domain."/index.php");
			// Always copy modified index.php (original cron_daily.php only copies if it not exists)
			copy($custom_conf_dir."/templates/awstats_index_subdir.php.master",$statsdomaindir."/index.php");

		

		} else {
			$app->log("No awstats statistics created. Either $awstats_pl or $awstats_buildstaticpages_pl is not owned by root user.",LOGLEVEL_WARN);
		}
		

		if(is_file($custom_stats_web_dir."/".$domain."/index.php")) {
			chown($custom_stats_web_dir."/".$domain."/index.php",$stats_site_system_user);
			chgrp($custom_stats_web_dir."/".$domain."/index.php",$stats_site_system_group);
		}

		if(is_file($statsdomaindir."/index.php")) {
			chown($statsdomaindir."/index.php",$stats_site_system_user);
			chgrp($statsdomaindir."/index.php",$stats_site_system_group);
		}


	}
	// END: if array aliases (extended version with awstats processing)
	///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////
// Finally compressing processed yesterdays logfile
if(@is_file($logfile_yesterday)) {
	exec("gzip $logfile_yesterday");
}

	

}

die("finished.\n");

?>
